#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>
#include "Sphere.h"
#include "ObjectBase.h"

Sphere::Sphere( dWorldID world, dSpaceID space, dReal r, dReal mass)
	: radius( r )
{
	body = dBodyCreate( world );
	geom = dCreateSphere( space, radius );

	dMass m;
	dMassSetSphere( &m, 1.0f, radius );
	dMassAdjust( &m, mass );
	dGeomSetBody( geom, body );

	init();
}

void Sphere::setColor(GLfloat r, GLfloat g, GLfloat b)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
}

void Sphere::renderInLocalFrame() const
{
	GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat low_shininess[] = {5.0};
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, low_shininess);

	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, color);
	glCallList(theSphere);
}


void Sphere::init()
{
    theSphere = glGenLists(1);
    glNewList(theSphere, GL_COMPILE);
    {
		glScalef((GLfloat)radius, (GLfloat)radius, (GLfloat)radius);
        draw( (GLfloat)radius, 20, 20 );
    }
    glEndList();
}

void Sphere::draw(GLfloat r, int lats, int longs)
{
    int i, j;
    for (i=0; i<=lats; i++) {
        GLfloat lat0 = (GLfloat)M_PI * (-0.5f + (GLfloat)(i-1)/(GLfloat)lats);
        GLfloat z0 = sin(lat0);
        GLfloat zr0 = cos(lat0);
 
        GLfloat lat1 = (GLfloat)M_PI * (-0.5f + (GLfloat)i/(GLfloat)lats);
        GLfloat z1 = sin(lat1);
        GLfloat zr1 = cos(lat1);
 
        glBegin(GL_QUAD_STRIP);
        for (j=0; j<=longs; j++) {
            GLfloat lng = 2.0f * (GLfloat)M_PI * (GLfloat)(j-1)/(GLfloat)longs;
            GLfloat x = cos(lng);
            GLfloat y = -sin(lng);
 
            glNormal3f(x * zr0, y * zr0, z0);
            glVertex3f(x * zr0, y * zr0, z0);
            glNormal3f(x * zr1, y * zr1, z1);
            glVertex3f(x * zr1, y * zr1, z1);
        }
        glEnd();
    }
}
 