#ifdef WIN32
#define _USE_MATH_DEFINES
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glut.h>
#include <ode/ode.h>
#include "Sphere.h"


static float view_xyz[3] = {0,-30,10};
static float view_hpr[3] = {90,-10,0};
static int ms_mode = 0;
static int ms_pre_x = 0;
static int ms_pre_y = 0;


GLfloat red[] = { 0.8, 0.2, 0.2, 1.0 };
GLfloat green[] = { 0.0, 1.0, 1.0, 1.0 };
GLfloat blue[] = { 0.0, 0.0, 1.0, 1.0 };
GLfloat yellow[] = { 1.0, 1.0, 0.0, 1.0 };

GLfloat light_pos[] = { 10.0f, 10.0f, 100.0f, 0.0 };


dWorldID world;
dSpaceID space;
dJointGroupID contactgroup;

Sphere *ball;


// Փˌop֐
static void nearCallback( void *data, dGeomID o1, dGeomID o2 )
{
	const int MAX_CONTACTS = 4;

	dBodyID b1 = dGeomGetBody( o1 ); // 1
	dBodyID b2 = dGeomGetBody( o2 ); // 2

	if ( b1 && b2 && dAreConnectedExcluding( b1, b2, dJointTypeContact ) )
	return; // ՓˑΏۂłȂ̂̏Փ˂͂͂

	dContact contact[MAX_CONTACTS];
	for ( int i=0; i<MAX_CONTACTS; i++ )
	{
		// ̓m̐ڐG̃p[^ݒ
		contact[i].surface.mode = dContactBounce | dContactSoftCFM;
		contact[i].surface.mu = dInfinity;   // CW
		contact[i].surface.bounce = 0.1;     // W
		contact[i].surface.soft_cfm = 0.01;  // CFMݒ
	}

	// Փˌo
	int numc = dCollide( o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof( dContact ) );
	if ( numc > 0 )
	{
		for ( int i=0; i<numc; i++ )
		{
			// Փ˂̔
			dJointID c = dJointCreateContact( world, contactgroup, contact+i );
						 // ڐGWCg
			dJointAttach( c, b1, b2 ); // ڐGWCgڑ
		}
	}
}



void drawGround()
{
    const float gsize = 12.0f;

    glDisable(GL_LIGHTING);
    glColor3f( 0.4, 0.3, 0.2 );

    glBegin( GL_TRIANGLE_STRIP );
    glNormal3f( 0,0,1 );
    glVertex3f(-gsize,-gsize,0);
    glVertex3f( gsize,-gsize,0);
    glVertex3f(-gsize, gsize,0);
    glVertex3f( gsize, gsize,0);
    glEnd();
}


void setCamera( float x, float y, float z, float h, float p, float r)
{
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glRotatef(90, 0, 0, 1);
    glRotatef(90, 0, 1, 0);
    glRotatef(r, 1, 0, 0);
    glRotatef(p, 0, 1, 0);
    glRotatef(h, 0, 0, -1);
    glTranslatef( -x, -y, -z );
}



void display(void)
{
	/////////////////////////////////////////////////////////////////////
	// ODE step

	dSpaceCollide( space, 0, &nearCallback );

	dWorldStep( world, 0.005 );

	dJointGroupEmpty( contactgroup ); 

	/////////////////////////////////////////////////////////////////////

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

	// camera
    setCamera( view_xyz[0], view_xyz[1], view_xyz[2], view_hpr[0], view_hpr[1], view_hpr[2] );

	// light
    glLightfv( GL_LIGHT0, GL_POSITION, light_pos);

	//
	// draw ground
	//
    glDisable(GL_LIGHTING);

    glColor3d(0.7, 0.7, 0.7); // grid line color
    glBegin(GL_LINES);
    for (int i = -10; i <= 10; i++) {
        glVertex3d((GLdouble)i, -10.0, 0);
        glVertex3d((GLdouble)i,  10.0, 0);
        glVertex3d(-10.0, (GLdouble)i, 0);
        glVertex3d( 10.0, (GLdouble)i, 0);
    }
    glEnd();
	
    glEnable (GL_POLYGON_OFFSET_FILL);
	glPolygonOffset (1.0, 0);
    drawGround();
	glDisable (GL_POLYGON_OFFSET_FILL);

	///////////////////////////////////////////////////////////////////////////
	// draw objects
	//

    glEnable(GL_LIGHTING);

	// draw ball
	ball->render();

	///////////////////////////////////////////////////////////////////////////

    glutSwapBuffers();
    //glFlush();
}



void timer(int value) {
	glutPostRedisplay();

	glutTimerFunc(17, timer , 0);
}


void resize(int w, int h)
{
    glViewport(0, 0, w, h);

    /* ϊs̐ݒ */
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //glFrustum( -(double)w/2.0, (double)w/2.0, -(double)h/2.0, (double)h/2.0, 1.0, 100.0 );
    gluPerspective(30.0, (double)w / (double)h, 1.0, 100.0);

    /* fr[ϊs̐ݒ */
    glMatrixMode(GL_MODELVIEW);
}


void mouse(int button, int state, int x, int y)
{
    switch (button) {
    case GLUT_LEFT_BUTTON:
        if (state == GLUT_DOWN) {
            ms_mode += 1; ms_pre_x = x; ms_pre_y = y;
        }
        else {
            ms_mode -= 1;
        }
        break;
    case GLUT_RIGHT_BUTTON:
        if (state == GLUT_DOWN) {
            ms_mode += 2; ms_pre_x = x; ms_pre_y = y;
        }
        else {
            ms_mode -= 2;
        }
        break;
    case GLUT_MIDDLE_BUTTON:
        if (state == GLUT_DOWN) {
            ms_mode += 4; ms_pre_x = x; ms_pre_y = y;
        }
        else {
            ms_mode -= 4;
        }
        break;
    default:
        break;
    }
}


void motion(int x, int y)
{
    int deltax = x - ms_pre_x;
    int deltay = y - ms_pre_y;

    float side = 0.01f * float(deltax);
    float fwd = (ms_mode==4) ? (0.01f*float(deltay)) : 0.0f;
    float s = (float) sin(view_hpr[0]*(M_PI/180.0));
    float c = (float) cos(view_hpr[0]*(M_PI/180.0));

    if (ms_mode == 1) {
        view_hpr[0] += float(deltax) * 0.5f;
        view_hpr[1] += float(deltay) * 0.5f;
    }
    else {
        view_xyz[0] += -s*side + c*fwd;
        view_xyz[1] += c*side + s*fwd;
        if (ms_mode==2 || ms_mode==5) view_xyz[2] += 0.01f * float(deltay);
    }
    for (int i=0; i<3; i++) {
        while (view_hpr[i]>180) view_hpr[i] -= 360;
        while (view_hpr[i]<-180) view_hpr[i] += 360;
    }
    ms_pre_x = x;
    ms_pre_y = y;
}


void keyboard(unsigned char key, int x, int y)
{
    switch (key) {
        case 'q':
        case 'Q':
        case '\033':  /* '\033'  ESC  ASCII R[h */
            exit(0);
            break;
        default:
            break;
    }
    glutPostRedisplay();
}


void init(void)
{
    glClearColor(0.5, 0.5, 1.0, 1.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
}

int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);

	glutInitWindowSize(600, 400);
    glutCreateWindow("test");

    glutDisplayFunc(display);
    glutReshapeFunc(resize);
    glutMouseFunc(mouse);
    glutKeyboardFunc(keyboard);
    glutMotionFunc(motion);
    //glutPassiveMotionFunc(motion);
    glutTimerFunc(100 , timer , 0);

    init();

	//////////////////////////////////////////////////////////////////////////////
	// ODE initialize
	dInitODE();

	world = dWorldCreate();
	dWorldSetGravity( world, 0.0, 0.0, -9.8 );

	space = dHashSpaceCreate( 0 );
	dCreatePlane( space, 0, 0, 1, 0 ); 

	contactgroup = dJointGroupCreate( 0 );

	// ball setting
	ball = new Sphere( world, space, 1.0f, 1.0f );
	ball->setPosition( 0.0, 0.0, 10.0 );
	ball->setColor( 1.0, 0.3, 0.3 );

	//////////////////////////////////////////////////////////////////////////////


	//
	// simulation
	//
    glutMainLoop();


	dWorldDestroy( world );
	dCloseODE();

    return 0;
}
